package banking;

/**
 *  A simple banking DatabaseAccountImpl class.
 *
 *@author     <a href="mailto:Sara.Bouchenak@inria.fr">Sara Bouchenak</a>
 *@version    1.0 - 13/01/2004
 */
public class DatabaseAccountImpl
	implements Account {
		
	/**
	 * The account number.
	 */
	private	int 			accountNumber;
	
	/**
	 * The underlying database.
	 */
	private DatabaseImpl	accountDb;
	
	/**
	 * Constructs an <code>DatabaseAccountImpl</code> with the specified number
	 * in a database identified by a URL, and admin name/password.
	 * 
	 * @param	number			The account number
	 * @param	customer		The customer name
	 * @param	dbUrl			The database URL
	 * @param	adminName 		The database administrator name
	 * @param	adminPassword	The database administrator password
	 * @param	balance			Amount of initial balanace
	 */
	public DatabaseAccountImpl(int number, String customer,
		String dbUrl, String adminName, String adminPassword,
		float balance)
	{
		this.accountDb		= new DatabaseImpl(dbUrl, 
			adminName, adminPassword, number, customer, balance);
		this.accountNumber	= number;
	}
	
	/**
	 * Returns the number of this account.
	 * 
	 * @return 	The account number.
	 */
	public int getAccountNumber()
	{
		return accountNumber;
	}
	
	/**
	 * Returns the associated database.
	 * 
	 * @return 	The database.
	 */
	public Database getDatabase()
	{
		return accountDb;
	}
	
	/**
	 * Returns the current balance of this account.
	 * 
	 * @return	The account balance.
	 * @throws	AccountException	If a problem occurs during
	 * this operation, e.g., a problem when accessing the underlying
	 * database if any.
	 */
	public float getBalance()
		throws AccountException
	{
		return accountDb.query(accountNumber);
	}
	
	/**
	 * Sets the balance of this account.
	 * 
	 * @param amount	The balance to be set
	 * @throws	AccountException	If a problem occurs during
	 * this operation, e.g., a problem when updating the underlying
	 * database if any.
	 */
	public void setBalance(float amount)
		throws AccountException
	{
		accountDb.update(accountNumber, amount);
	}
	
	/**
	 * Adds an amount to the current balance of this account.
	 * 
	 * @param 	amount	The amount to add.
	 * @throws	AccountException	If a problem occurs during
	 * this operation, e.g., a problem when accessing/updating 
	 * the underlying database if any.
	 */
	public void credit(float amount)
		throws AccountException
	{
		setBalance(getBalance() + amount);
	}
	
	/**
	 * Deducts an amount from the current balance of this account.
	 * 
	 * @param 	amount				The amount to be deducted.
	 * @throws 	AccountException	In case the amount to deduct 
	 * is greater than the current balance, or if a problem occurs 
	 * when querying/updating the underlying database if any.
	 */
	public void debit(float amount)
		throws AccountException
	{
		float balance = getBalance();
		
		if (balance < amount)
		{
			throw (new InsufficientBalanceException("Total balance not sufficient"));
		}
		else
		{
			setBalance(balance - amount);
		}
	}
	
}